class injector:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self):
        self.registry = {}

    def register(self, key):
        def decorator(func):
            self.registry[key] = func
            return func
        return decorator

    def registerClass(self, key, instance):
        self.registry[key] = instance

    def inject(self, key):
        return self.registry.get(key)


import base64

class dataEncryptor:
    def __init__(self, key_manager):
        self.key_manager = key_manager

    def encrypt(self, source, file_or_string):
        import os
        key = self.key_manager.key
        if isinstance(source, str):
            data = source.encode()
        elif isinstance(source, os.__file__):
            with open(source, 'rb') as f:
                data = f.read()
        else:
            raise ValueError("Invalid source type. Must be a file or string.")

        encrypted_data = self._xor_encrypt(data, key)
        if file_or_string == 'file':
            encrypted_file = source + '.enc'
            with open(encrypted_file, 'wb') as f:
                f.write(encrypted_data)
            return encrypted_file
        elif file_or_string == 'string':
            return base64.b64encode(encrypted_data).decode()
        else:
            raise ValueError("Invalid file_or_string value. Must be 'file' or 'string'.")

    def decrypt(self, source, file_or_string):
        key = self.key_manager.key
        if file_or_string == 'file':
            decrypted_file = source[:-4]  # Remove the '.enc' extension
            with open(source, 'rb') as f:
                encrypted_data = f.read()
        elif file_or_string == 'string':
            encrypted_data = base64.b64decode(source.encode())
        else:
            raise ValueError("Invalid file_or_string value. Must be 'file' or 'string'.")

        decrypted_data = self._xor_encrypt(encrypted_data, key)
        if file_or_string == 'file':
            with open(decrypted_file, 'wb') as f:
                f.write(decrypted_data)
            return decrypted_file
        elif file_or_string == 'string':
            return decrypted_data.decode()
        else:
            raise ValueError("Invalid file_or_string value. Must be 'file' or 'string'.")

    def _xor_encrypt(self, data, key):
        encrypted_data = bytearray()
        key_length = len(key)
        for i, byte in enumerate(data):
            encrypted_data.append(byte ^ key[i % key_length])
        return encrypted_data


# @singleton
# Make a class a singleton. Only one instance of the class will be created and returned.

def singleton(cls):
    instances = {}

    def wrapper(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return wrapper


# @private
# Make a class private and raise an exception if it is accessed from outside the module.

def private(cls):
    def wrapper(*args, **kwargs):
        raise Exception("This class is private and cannot be accessed from outside the module!")
    return wrapper

# @static
# Make a class static and raise an exception if it is instantiated. (EXPERIMENTAL)

def static(cls):
    def wrapper(*args, **kwargs):
        raise Exception("This class is static and cannot be instantiated!")
    return wrapper

# @threaded
# Make a class threaded and return a thread object. (EXPERIMENTAL)

def threaded(cls):
    import threading

    def wrapper(*args, **kwargs):
        return threading.Thread(target=cls, args=args, kwargs=kwargs)
    return wrapper

# @trycatch
# Make a piece of code try-catchable without using try-catch blocks.
def trycatch(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            print(f"An error occurred: {e}")
    return wrapper

# @loop(condition)
# Make a function loop until a condition is met. (EXPERIMENTAL)
def loop(condition_func):
    def decorator(func):
        def wrapper(*args, **kwargs):
            while condition_func():
                func(*args, **kwargs)
        return wrapper
    return decorator

# @deprecated
# Mark a function as deprecated and print a log when its used for the first time, also raise a warning and show in the editor as a warning.
def deprecated(func):
    import warnings

    def wrapper(*args, **kwargs):
        warnings.warn(f"Function {func.__name__} is deprecated and will be removed in the future.", DeprecationWarning, stacklevel=2)
        return func(*args, **kwargs)
    return wrapper

# @experimental
# Mark a function as experimental and print a log when its used for the first time, also raise a warning and show in the editor as a warning.
def experimental(func):
    import warnings

    def wrapper(*args, **kwargs):
        warnings.warn(f"Function {func.__name__} is experimental and may not work as expected.", UserWarning, stacklevel=2)
        return func(*args, **kwargs)
    return wrapper

# @notnull
# Make a function not return None, undefined or null.
def notnull(func):
    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        if result is None:
            raise Exception("Function returned None!")
        return result
    return wrapper

# @delay(seconds)
# Delay a function by a number of seconds.
def delay(seconds):
    import time

    def decorator(func):
        def wrapper(*args, **kwargs):
            time.sleep(seconds)
            return func(*args, **kwargs)
        return wrapper
    return decorator

# @timeout(seconds)
# Timeout a function after a number of seconds.
class TimeoutException(Exception):
    pass

def timeout(seconds):
    from threading import Thread
    def decorator(func):
        def wrapper(*args, **kwargs):
            res = [TimeoutException('Function timed out!')]
            def target():
                try:
                    res[0] = func(*args, **kwargs)
                except Exception as e:
                    res[0] = e
            t = Thread(target=target)
            t.start()
            t.join(seconds)
            if isinstance(res[0], BaseException):
                raise res[0]
            return res[0]
        return wrapper
    return decorator

# @retry(attempts/-1, delay)
# Retry a function a number of times with a delay between each attempt.
def retry(attempts, delay):
    import time
    def decorator(func):
        def wrapper(*args, **kwargs):
            for i in range(attempts):
                try:
                    return func(*args, **kwargs)
                except Exception as e:
                    print(f"Attempt {i} failed: {e}")
                    time.sleep(delay)
            raise Exception("Function failed after all attempts!")
        return wrapper
    return decorator

# @log
# Log a function's arguments and return value.
def log(func):
    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        print(f"Function {func.__name__} called with args: {args} and kwargs: {kwargs}, returned: {result}")
        return result
    return wrapper


import datetime

class Queue:
    def __init__(self):
        self.queue = []
        self.log = []

    def add_item(self, item):
        self.queue.append(item)
        self.log_action('ADD', item)

    def remove_item(self):
        if self.queue:
            item = self.queue.pop(0)
            self.log_action('REMOVE', item)
            return item
        else:
            return None

    def check_item(self):
        if self.queue:
            item = self.queue[0]
            self.log_action('CHECK', item)
            return item
        else:
            return None

    def clear_queue(self):
        self.queue = []
        self.log_action('CLEAR', 'Queue cleared')

    def log_action(self, action, item):
        timestamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        log_entry = f'[{timestamp}] QUEUE {action} TO {item} ({len(self.queue)})'
        self.log.append(log_entry)

    def print_log(self):
        for log_entry in self.log:
            print(log_entry)


from ..textUtils import textUtils

class Logger:
    def __init__(self, prefix = None, debug=False, log = None, format = "{event} {white}| {message}"):
        self.prefix = prefix
        self.log = log
        self.format = format
        self.debug = debug
        self.text_utils = textUtils()

    def  __toLogFile(self, message):
        if self.log is not None:
            with open(self.log, "a") as file:
                file.write(message + "\n")

    def __log(self, event, message):
        if self.prefix is not None:
            event = self.prefix + " " + event
        message = self.text_utils.format(message)
        formatted_message = f"{event} {message}"
        print(formatted_message)
        self.__toLogFile(formatted_message)
    
    def info(self, message):
        self.__log("INFO", message)

    def warning(self, message):
        self.__log("WARNING", message)

    def error(self, message):
        self.__log("ERROR", message)

    def success(self, message):
        self.__log("SUCCESS", message)

    def debug(self, message):
        if self.debug:
            self.__log("DEBUG", message)

    def announce(self, message):
        self.__log("ANNOUNCE", message)


class textUtils:
    def color(color, text):
        pattern = "" # {0} is the text
        match color:
            case "red":
                pattern = "\033[91m{0}\033[00m"
            case "green":
                pattern = "\033[92m{0}\033[00m"
            case "yellow":
                pattern = "\033[93m{0}\033[00m"
            case "blue":
                pattern = "\033[94m{0}\033[00m"
            case "purple":
                pattern = "\033[95m{0}\033[00m"
            case "cyan":
                pattern = "\033[96m{0}\033[00m"
            case "white":
                pattern = "\033[97m{0}\033[00m"
            case "dark_red":
                pattern = "\033[31m{0}\033[00m"
            case "dark_green":
                pattern = "\033[32m{0}\033[00m"
            case "gold":
                pattern = "\033[33m{0}\033[00m"
            case "gray":
                pattern = "\033[37m{0}\033[00m"
            case "dark_gray":
                pattern = "\033[90m{0}\033[00m"
            case "black":   
                pattern = "\033[30m{0}\033[00m"
            case "reset":
                pattern = "\033[00m{0}\033[00m"
            case _:
                pattern = "{0}"
        return pattern.format(text)
    def decorate(decoration, text):
        match decoration:
            case "bold":
                return "\033[1m" + text + "\033[0m"
            case "underline":
                return "\033[4m" + text + "\033[0m"
            case "italic":
                return "\033[3m" + text + "\033[0m"
            case _:
                return text
        
    def format(text):
        aliases = {
            "bold": "\033[1m",
            "underline": "\033[4m",
            "italic": "\033[3m",
            "red": "\033[91m",
            "green": "\033[92m",
            "yellow": "\033[93m",
            "blue": "\033[94m",
            "purple": "\033[95m",
            "cyan": "\033[96m",
            "white": "\033[97m",
            "dark_red": "\033[31m",
            "dark_green": "\033[32m",
            "gold": "\033[33m",
            "gray": "\033[37m",
            "dark_gray": "\033[90m",
            "black": "\033[30m",
            "reset": "\033[0m"
        }

        for key, value in aliases.items():
            text = text.replace(key, value)

        return text + "\033[0m"  # reset at the end


class config:
    def __init__(self, configContainer):
        self.configContainer = configContainer
        self.registeredSettings = {}

    # @setting (Decorator)
    """
    USAGE EXAMPLE:

    @setting()
    def exitMsg():
        return "Program has been terminated."

    - This would register the setting 'exitMsg' with the type 'str' into the registeredSettings dictionary, and with the default value of "Program has been terminated." in case the key has not still generated on the config file.

    """
    # Gets the name of the setting and the type of the variable it returns and stores it into the registeredSettings dictionary.
    def setting(self, type=str):
        def decorator(func):
            # Store the function name and its return value
            self.registeredSettings[func.__name__] = func()
            # Register the setting in the configContainer
            self.configContainer.setValue(func.__name__, func())
        return decorator


import re
from random import randint, choice, shuffle
import base64
import string

class keyManager():
    def __init__(self):
        cache = str(randint(0, 2550))+str(randint(0, 2550))+str(randint(0, 2550))
        self.encryptBase = base64.b64encode(cache.encode())
        self.key = self.shuffle_string(self.replace_non_alnum(self.__keyGen())).encode()
    
    def __keyGen(self):
        cache = 0
        for char in self.encryptBase:
            for i in range(0, 255):
                cache += int(char) ^ i
        
        cache = str(int(cache)/2).replace(".", "").replace(",", "")

        for char in cache:
            if randint(0, 10) == 5:
                cache = cache.replace(char, str(randint(0, 9)))

        for char in cache:
            if int(char) > 2:
                lower_bound = int(str(int(char)/2).replace('.', ''))
                upper_bound = int(char)
                if lower_bound > upper_bound:
                    lower_bound, upper_bound = upper_bound, lower_bound
                cache += str(randint(lower_bound, upper_bound))
        
        return cache

    def shift_char(self, c):
        return chr((ord(c) + 10) % 256)

    def shift_string(self, s):
        return ''.join(self.shift_char(c) for c in s)

    def replace_non_alnum(self, s):
        s = self.shift_string(s)
        s = re.sub(r'\W', ''.join(choice(string.ascii_letters + string.digits + string.punctuation) for _ in range(10)), s)
        return s

    def shuffle_string(self, s):
        s_list = list(s)
        shuffle(s_list)
        return ''.join(s_list)
    
key = keyManager().key
print(key)


import os
import json
import yaml
import logging

class configContainer:
    def __init__(self, path, filename, type):
        self.path = path
        self.filename = filename
        self.type = type
        self.data = {}
        self.load()

    def load(self):
        if self.type == "json":
            self.loadJSON()
        elif self.type == "yaml":
            self.loadYAML()
        else:
            raise TypeError(f"Type '{self.type}' is not supported, use 'json' or 'yaml' instead.")

    def loadJSON(self):
        os.makedirs(self.path, exist_ok=True)
        file_path = os.path.join(self.path, f"{self.filename}.json")
        if not os.path.isfile(file_path):
            with open(file_path, "w") as file:
                json.dump({}, file)
        with open(file_path, "r") as file:
            self.data = json.load(file) or {}

    def loadYAML(self):
        os.makedirs(self.path, exist_ok=True)
        file_path = os.path.join(self.path, f"{self.filename}.yaml")
        if not os.path.isfile(file_path):
            with open(file_path, "w") as file:
                file.write("{}")
        with open(file_path, "r") as file:
            try:
                self.data = yaml.safe_load(file) or {}
            except Exception as e:
                logging.error(f"Failed to load YAML file {file_path}: {e}")
                self.data = {}

    def registerValues(self, config):
        for key, value in config.items():
            self.__register(key, value)

    def __register(self, key, value):
        if key not in self.data:
            self.data[key] = value
            self.__save()

    def __save(self):
        if self.type == "json":
            self.__saveJSON()
        elif self.type == "yaml":
            self.__saveYAML()
        else:
            raise TypeError(f"Type '{self.type}' is not supported, use 'json' or 'yaml' instead.")

    def __saveJSON(self):
        with open(os.path.join(self.path, f"{self.filename}.json"), "w") as file:
            json.dump(self.data, file, indent=4)

    def __saveYAML(self):
        with open(os.path.join(self.path, f"{self.filename}.yaml"), "w") as file:
            yaml.dump(self.data, file)

    def get(self, key):
        return self.data.get(key)

    def getValue(self, key):
        value = self.data.get(key)
        return value['value'] if isinstance(value, dict) else value

    def setValue(self, key, value):
        self.data[key] = value
        self.__save()

    def changeType(self, key, type):
        self.data[key] = type(self.data[key])
        self.__save()

    def reload(self):
        self.load()
        return self.data


from setuptools import setup, find_packages

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
name='decoutilities',
version='0.2.8',
author='Hugo Torres',
author_email='contact@redactado.es',
description='Enhance the readability of your code with decorators and simplify the creation of configuration files.',
packages=find_packages(include=['decoutilities', 'decoutilities.*']),
classifiers=[
'Programming Language :: Python :: 3',
'License :: OSI Approved :: MIT License',
'Operating System :: OS Independent',
],
python_requires='>=3.6',
long_description=long_description,
long_description_content_type='text/markdown'
)